package keystash.gui;

import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.spec.InvalidKeySpecException;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import javax.crypto.BadPaddingException;
import javax.crypto.IllegalBlockSizeException;
import javax.crypto.NoSuchPaddingException;
import keystash.KeyStashGUI;
import static keystash.KeyStashGUI.accountMenuItem;
import static keystash.KeyStashGUI.backgroundMenu;
import static keystash.KeyStashGUI.confirmNewAccountAttemptLabel;
import static keystash.KeyStashGUI.confirmNewAccountAttemptValueLabel;
import static keystash.KeyStashGUI.confirmNewAccountEntropyValue;
import static keystash.KeyStashGUI.confirmNewAccountFrame;
import static keystash.KeyStashGUI.confirmNewAccountGenerateNewPhraseButton;
import static keystash.KeyStashGUI.confirmNewAccountIncludeLettersField;
import static keystash.KeyStashGUI.confirmNewAccountLengthValue;
import static keystash.KeyStashGUI.confirmNewAccountOKButton;
import static keystash.KeyStashGUI.confirmNewAccountPhraseField;
import static keystash.KeyStashGUI.confirmNewAccountSearchAddressToggleButton;
import static keystash.KeyStashGUI.confirmNewAccountShowAddressRsLabel;
import static keystash.KeyStashGUI.confirmNewAccountShowPhraseCheckBox;
import static keystash.KeyStashGUI.enableApiMenuItem;
import static keystash.KeyStashGUI.globalPassword;
import static keystash.KeyStashGUI.guiShowWarnings;
import static keystash.KeyStashGUI.secret;
import keystash.crypto.Aes;
import keystash.crypto.Crypto;
import keystash.crypto.PhraseGenerator;
import keystash.http.Refresh;
import keystash.util.Constants;
import keystash.util.Convert;
import keystash.util.Date;
import keystash.util.Db;
import keystash.util.Memory;
import keystash.util.PasswordStrength;
import org.apache.commons.codec.DecoderException;

public class ConfirmNewAccount {

    public static void saveAccount() {

        char[] checkPassword = confirmNewAccountPhraseField.getPassword();
        int passLength = checkPassword.length;
        if (passLength < 35) {
            
            Dialog.showDialog("Minimum Length of phrase must be over 35 characters.", "Note");
            
        }
        else {
            if (enableApiMenuItem.isSelected()) {
                
                backgroundMenu = 3;
                try {
                    Thread.sleep(1000);
                } catch (InterruptedException ex) {
                    Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
                }
                
            }
            int accountId = Db.countAccountsWallet("keystash_wallet") + 1;
            char[] genSalt = null;
            try {
                genSalt = Aes.generateSaltOrIv(Constants.saltAndIvSize[0]);
            } catch (NoSuchAlgorithmException | NoSuchProviderException ex) {
                Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            char[] tmpSalt = genSalt;
            char[] genIV = null;
            try {
                genIV = Aes.generateSaltOrIv(Constants.saltAndIvSize[1]);
            } catch (NoSuchAlgorithmException | NoSuchProviderException ex) {
                Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            char[] tmpIV = genIV;

            char[] cipherPassPhrase = null;
            try {
                cipherPassPhrase = Aes.encrypt(String.valueOf(secret), String.valueOf(globalPassword), new String(tmpSalt), new String(tmpIV));
            } catch (NoSuchAlgorithmException | InvalidKeySpecException | NoSuchPaddingException | IllegalBlockSizeException | BadPaddingException | InvalidKeyException | InvalidAlgorithmParameterException | DecoderException ex) {
                Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
            }

            byte[] secretPhrase = Crypto.getPublicKey(String.valueOf(secret));
            Long secretPhraseConvert = Convert.getId(secretPhrase);
            String publicKey = Convert.toHexString(secretPhrase);

            String addressClassic = Convert.toUnsignedLong(secretPhraseConvert);
            String addressRs = Convert.rsAccount(secretPhraseConvert);

            String label = Constants.accountLabel;
            int secretIteration = (Convert.stringToInt(Db.selectFromSettings("secret_iteration", "keystash_settings", "settings_id", 1)));
            int secretKeySize = (Convert.stringToInt(Db.selectFromSettings("secret_key_size", "keystash_settings", "settings_id", 1)));
            int shareAccount = (Convert.stringToInt(Db.selectFromSettings("share_account", "keystash_settings", "settings_id", 1)));
            String accountType = Db.selectFromSettings("account_type", "keystash_settings", "settings_id", 1);

            String createdTimestamp = Date.getDate();

            Db.walletInsertNewAccount(accountId, addressClassic, addressRs, publicKey, label, new String(cipherPassPhrase), new String(tmpSalt), new String(tmpIV), secretIteration, secretKeySize, shareAccount, accountType, createdTimestamp);

            TableContent.deleteGuiTables();
            TableContent.getTable();

            Refresh.refreshApi();

            guiShowWarnings = Convert.stringToBoolean(Db.selectFromSettings("show_warnings", "keystash_settings", "settings_id", 1));
            if (guiShowWarnings) {
                
                Dialog.showDialog("Always backup your wallet after changes on a secure place!", "Note");
                
            }

            confirmNewAccountFrame.setVisible(false);
            accountMenuItem.setEnabled(true);
            MenuItem.enableMenu(true);
        }

        if (enableApiMenuItem.isSelected()) {
            
            try {
                Thread.sleep(1000);
            } catch (InterruptedException ex) {
                Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
            }
            backgroundMenu = 2;
            
        }
        Memory.clearPasswordMemory(checkPassword);
        Memory.clearPasswordMemory(secret);
        Memory.clearPasswordMemory(globalPassword);
    }

    public static void showPhrase() {

        boolean showPhrase = confirmNewAccountShowPhraseCheckBox.isSelected();

        if (showPhrase) {
            
            confirmNewAccountPhraseField.setEchoChar((char) 0);
            
        }
        else {
            
            confirmNewAccountPhraseField.setEchoChar('*');
            
        }

    }

    public static void generateNewPhrase() {

        char[] myPhrase = null;
        try {
            myPhrase = PhraseGenerator.generatePhrase();
        } catch (NoSuchAlgorithmException | NoSuchProviderException ex) {
            Logger.getLogger(KeyStashGUI.class.getName()).log(Level.SEVERE, null, ex);
        }
        secret = myPhrase;
        byte[] secretPhrase = Crypto.getPublicKey(new String(secret));
        Long secretPhraseConvert = Convert.getId(secretPhrase);

        String addressRs = Convert.rsAccount(secretPhraseConvert);
        confirmNewAccountShowAddressRsLabel.setText(addressRs);
        confirmNewAccountPhraseField.setText(new String(myPhrase));
        int passLength = secret.length;
        String passLengthString = String.valueOf(passLength);
        double shannonEntropy = PasswordStrength.getShannonEntropy(new String(secret));
        String shannonEntropyString = String.valueOf(shannonEntropy);
        confirmNewAccountEntropyValue.setText(shannonEntropyString);
        confirmNewAccountLengthValue.setText(passLengthString);

    }

    public static void searchAddress() {

        boolean isToogled = confirmNewAccountSearchAddressToggleButton.isSelected();
        String r = ".*[*$-+.,10#?_&=!%{}/]+.*";
        Matcher m = Pattern.compile(r).matcher(confirmNewAccountIncludeLettersField.getText());

        if (isToogled) {
            
            if (confirmNewAccountIncludeLettersField.getText().equals("NXT-     ")) {
                Dialog.showDialog("Searchfield is empty!", "Note");
                confirmNewAccountSearchAddressToggleButton.setSelected(false);
                
            }
            else if (m.find()) {
                
                Dialog.showDialog("Wrong characters!", "Note");
                confirmNewAccountSearchAddressToggleButton.setSelected(false);
                
            }
            else {

                confirmNewAccountAttemptLabel.setEnabled(true);
                confirmNewAccountAttemptValueLabel.setEnabled(true);
                confirmNewAccountGenerateNewPhraseButton.setEnabled(false);
                confirmNewAccountOKButton.setEnabled(false);
                confirmNewAccountIncludeLettersField.setEnabled(false);
                backgroundMenu = 4;
                
            }
        }
        else {
            
            confirmNewAccountAttemptLabel.setEnabled(false);
            confirmNewAccountAttemptValueLabel.setEnabled(false);
            confirmNewAccountGenerateNewPhraseButton.setEnabled(true);
            confirmNewAccountOKButton.setEnabled(true);
            confirmNewAccountIncludeLettersField.setEnabled(true);
            
        }

    }

    public static void closeFrame() {

        confirmNewAccountAttemptLabel.setEnabled(false);
        confirmNewAccountAttemptValueLabel.setEnabled(false);
        confirmNewAccountGenerateNewPhraseButton.setEnabled(true);
        confirmNewAccountOKButton.setEnabled(true);
        confirmNewAccountIncludeLettersField.setEnabled(true);
        confirmNewAccountSearchAddressToggleButton.setSelected(false);
        backgroundMenu = 0;
        confirmNewAccountFrame.setVisible(false);
        accountMenuItem.setEnabled(true);
        confirmNewAccountFrame.dispose();
        MenuItem.enableMenu(true);

    }

}
